@rem = ' This is a perl script for NT and UNIX.
@echo off
goto endofperl
@rem ';
# -*- tab-width: 4 -*-
#---------------------------------------------------------------------------+
# Licensed Materials - Property of IBM
# Rational ClearCase
# Copyright IBM Corp. 1998, 2006 All Rights Reserved
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract
# with IBM Corp.
#---------------------------------------------------------------------------+

###########################################################################
# syncmgr_export_list.bat
# usage: syncmgr_export_list - run with "-help" for detailed usage.
#
# This script is intended to be run periodically by the ClearCase
# Job Scheduler to send synchronization update packets to 
# other replica(s) of one or more vob(s).
#
# It may also be used to initiate sync export activity ad-hoc from the
# command line with a variety of options.
# 
# Run the script with the "-help" switch for description of the parameters.
###########################################################################


use strict;
use Getopt::Long;

sub dbgprint;    # predeclare it; routine in require file
sub qprint;      # predeclare...
sub print2;      # predeclare...

my $thisscript = $0;
if ($thisscript =~ m/(.*)(\.bat)$/) {
    $thisscript = $1;
}
use ClearCase::CurrentVersion;

$CFG::including_script = $thisscript;

my $pkg_name = 'syncmgr_export_list';
$CFG::verbose = ($ENV{TRACE_SUBSYS} && ($ENV{TRACE_SUBSYS} =~ m/$pkg_name/));
$CFG::failcount = 0;

my $reptosend = "";

# force debugging msgs by uncommenting the following.....
#$CFG::verbose = 1;

# set script version to test for mismatch against the syncmgr_common.pl version.
$CFG::MS_script_version = "7.0.0.0_syncmgr";

# get the machine's architecture 
($CFG::NT, $CFG::UNIX) = GetArchitecture();

# CLEARCASEHOME e.v. should always be set on NT when called by ccperlpp.pl,
# so exit if it's not set. On UNIX, assume /opt/rational/clearcase if user
# hasn't explicitly set CLEARCASEHOME (or deprecated ATRIAHOME).
#
if ($CFG::UNIX) {
    my $producthome = &ClearCase::CurrentVersion::SetProductHomeEV;
    chdir $producthome;
    push(@INC, "$producthome/config/scheduler/tasks");
} else {
    my $producthome = $ENV{CLEARCASEHOME} or 
                      die "CLEARCASEHOME environment variable not set\n";
    chdir $producthome;
    push(@INC, "$producthome\\config\\scheduler\\tasks");
    $thisscript = $pkg_name;
}

require 'syncmgr_common.pl';
require 5.8.6;


if (!$CFG::MScommon_version) {
    $CFG::MScommon_version = "(unknown older version)";
}

# check for mismatched versions - syncmgr_common.pl should set 
# $CFG::MScommon_version; if not set or unequal, may indicate that an 
# older/newer version is being included improperly.

if ($CFG::MScommon_version ne $CFG::MS_script_version) {
    die "Version mismatch between require file 'syncmgr_common.pl' " .
        "(version: ${CFG::MScommon_version}) and \n" .
        "the MultiSite export script which includes it, " .
        "'syncmgr_export_list.bat'" .
        " (version: ${CFG::MS_script_version}).\n" . 
        "Ensure that old versions of the script/require file are not in use.\n";
}

$CFG::failcount = 0;

$| = 1 if $CFG::verbose;

my $arch = $CFG::NT ? "Windows" : "UNIX";
dbgprint "running on architecture $arch\n";

###########################################################################           
# main
#
# get command line options figured out...
dbgprint "ARGV: " . join(' ',@ARGV) . "\n";

my $multitool_cmd = "$CFG::MultiTool syncreplica -export ";

my $usage = 
    "$thisscript [-[no]compress] [-logdir logdir] [-help]\n" .
    "    [-fship | -ship] [-lockwait minutes]\n" .
    "    [-view viewtag] [-quiet mode] [-workdir workdir] [-nblocking]\n" .
    "    [-maxsize maxsize[k|m|g]] [-limit limit]\n" . 
    "    [-trace] [-[no]poll] [-sclass sclass] [-update]\n" .
    "    [-iterate tries] [-wait seconds]\n" .
    "    { -all | -replicas replica-list [script-file] | script-file }\n\n" .
    "    The shortest umambiguous abbreviation for switches may be used.\n" .
    "    More help is available by using the '-help' command-line switch.\n";

my $helpmsg = <<ENDHELP;
=========================================================================
"syncmgr_export_list" exports one or more VOB replicas. The VOBs/replicas to
be exported, as well as options for the operation, may be specified either
on the command line, or via a script-file whose name is specified as a
parameter, or a combination of these two methods.

NOTE: this script is intended to be used if you are using "enhanced" mode 
MultiSite with the synchronization manager (syncmgr). If you use "standard"
mode MultiSite, use the "sync_export_list" script instead.

A logfile is created during execution, but is normally deleted if the script 
executes successfully, and is kept if a failure is detected.

Setting the environment variable TRACE_SUBSYS to "syncmgr_export_list" will 
turn on all debugging print statements (verbose mode). Using the "-trace" 
command line switch will print a less-verbose set of messages. Using either 
tracing method will retain the logfile even when the script is successful.

$usage
Command line options:

$CFG::help_parm_help
$CFG::logdir_parm_help
$CFG::workdir_parm_help
$CFG::quiet_parm_help
$CFG::trace_parm_help
$CFG::view_parm_help
    -[no]compress       Turns compression of exported packets on/off.
                        (Default is "-nocompress").

    -all                Export from all local replicas. This is mutually
                        exclusive with the "-replicas" parameter and use of
                        scriptfiles.

    -fship              Force shipping of packets immediately (default).

    -ship               Overrides -fship (packets will wait until shipping
                        server runs).

    -maxsize size       Max packet size, with number followed by size letter,
                        e.g. "500k", "20m", "1.5g". If not specified, MultiSite
                        will use the default size from shipping.conf (UNIX)
                        or the Control Panel applet (NT).

    -limit max-packets  Maximum number of packets to generate. (Default:
                        no limit).

    -sclass class       Storage class to use. (Default: use default sclass).

    -[no]update         Update this replica's epoch table from remote replica
                        before performing export. (Default: "-noupdate").

    -[no]poll           Perform "shipping_server -poll" before exporting.
                        Uses "-sclass" if specified. (Default: "-nopoll").

    -lockwait minutes   The amount of time (in minutes) to wait on VOB 
			locks that block VOB writes. Minutes must be > 0.
			(Default: no waiting).
			NOTE: If VOB locked and lockwait NOT set, old
			values in epoch table may cause unnecessary packet
			transfers on subsequent exports.

                        The CLEARCASE_VOBLOCKWAIT e.v. is ignored by this 
                        script, use this parameter instead.

    -nblocking          When executing multitool from the script, continue
                        on with the script after queueing the multitool request.
                        The script may complete before the export jobs have
                        finished. The default is for multitool to "wait 
                        forever" until an export completes before allowing the
                        script to continue.

    -replicas repnames  Specify a VOB and replica(s) to use. You may specify
                        a single replica: rep1@\\myvob, or multiple replicas
                        as: rep1@\\myvob,rep2,rep3. If you specify only the
                        VOB as: \\myvob, syncs will be generated for ALL
                        of this replica's siblings. If you wish to use more
                        than one VOB, use a 'script-file' as described below.
                        NOTE: if you specify multiple replicas, you must not 
                        leave any white space in the parameter.

    -iterate num_tries  Maximum number of tries to get all exports to complete
                        successfully. A retry may be successful for various
                        reasons, such as a lock being released or a VOB
                        mounted. (Default: 1, indicating that failed exports
                        will NOT be retried).

    -wait seconds       Useful if '-iterate' is specified. The amount of wait
                        time (in seconds) before re-attempting failed exports.
                        (Default: 30 seconds).

    script-file         Path to file containing a set of directives to 
                        execute. Must be the final parameter specified on
                        the command line. (Default is no script-file).

                        Script-file may contain the following directives;
                        directives with values will have the value immediately
                        following a colon (":"). All directives are processed
                        in sequence, and those which affect export settings
                        (i.e. all except 'replicas') remain in effect for all
                        subsequent 'replicas' directives unless modified.
                        
                        [no]compress - set/unset export compression.
                        maxsize:<size-specifier> - set max packet size.
                        ship -  export with "-ship" qualifier.
                        fship - export with "-fship" qualifier.
                        limit:<number> - set max number of packets
                            to export per replica.
                        [no]update - set/unset whether epoch table is updated
                            prior to export.
                        sclass:<storage-class> - set a different storage class;
                            leave value blank to unset.
                        replicas:<list> - initiates the export for list of
                            VOB replicas. See above for format of list.
                        lockwait:<number> - set minutes to wait for VOB locks
                        view:<viewtag> - view tag
                        [no]nblocking - do/do not block on multitool commands
                        workdir:<workdir> - assign new working directory

The script configuration options file, located at 
'${CFG::script_configfile}',
is no longer used in this version of the MultiSite import/export scripts,
and settings in this file will be ignored.
=========================================================================
ENDHELP

# delete local voblockwait EV - user must specify "-lockwait"
RemoveLockwaitEV();

my $do_compress = 0;
my $logdir = '';
my $help = 0;
my $all_local = 0;
my $fshipflg = 1;
my $shipflg = 0;
my $lockwait = 0;
my $view = '';
$CFG::traceflg = 0;
my $maxsize = '';
my $limit = 0;
my $poll = 0;
my $updateflg = 0;
$CFG::quietmode = 0;
my $reps = '';
my $sleeptime = 30;
my $workdir = '';
my $num_tries = 1;  # default: only one try
my $sclass = '';

my $nblocking = 0;

my $st = GetOptions("compress!" => \$do_compress,
                 "logdir=s" => \$logdir,
                 "help" => \$help,
                 "all" => \$all_local,
                 "fship" => \$fshipflg,
                 "ship" => \$shipflg,
                 "lockwait=i" => \$lockwait,
                 "view=s" => \$view, 
                 "trace!" => \$CFG::traceflg,
                 "maxsize=s" => \$maxsize,
                 "limit=i" => \$limit,
                 "poll!" => \$poll,
                 "update!" => \$updateflg,
                 "quiet=i" => \$CFG::quietmode,
                 "replicas=s" => \$reps,
                 "wait=i"     => \$sleeptime,
                 "nblocking!" => \$nblocking, 
                 "workdir=s"   => \$workdir,
                 "iterate=i"  => \$num_tries,
                 "sclass=s" => \$sclass);

if (!$st) {
    qprint "ERROR: error getting command line options.\nUsage: $usage\n";
    exit 1;
}

if ($help) {
    qprint "$helpmsg\n";
    exit 0;
}

if (scalar @ARGV > 1) {
    qprint "ERROR: too many script files specified in cmd line!\n";
    exit 1;
}

if ($all_local && ((scalar @ARGV > 1) || $reps)) {
    qprint "ERROR: '-all' may not be specified with '-replicas' or " .
        "a scriptfile.\n";
    exit 1;
}

if ($shipflg) {  # "-ship" will override "-fship"
    $fshipflg = 0;
}

my $export_vob = '';
my @replicas;

if ($reps) {
    ($export_vob, @replicas) = GetReplicas($reps);
    if (!$export_vob) {
        qprint "ERROR: error parsing '-replicas' argument.\n$usage\n";
        exit 1;
    }
print "    vob: $export_vob\n    replicas: " . join(" ",@replicas) . "\n"
    if ($CFG::verbose || $CFG::traceflg);
}

my $scriptfile = $ARGV[0] if (scalar @ARGV);  # script/config file specified

#
# Validate that the log directory actually exists...
#
$logdir = ValidateLogDir($logdir);

#
# Redirect STDERR to a uniquely named log file
#
my $logname = RedirectSTDERRtoLog ($logdir, 'send');

print STDERR "\n============================================================\n"
    . "Beginning execution of MultiSite export script '$thisscript' at " .
    MakeTimestamp() . ".\n\n";

# begin by forcing any existing packets to ship if "-poll" specified on cmd
# line
if ($poll) {
    my $sclass_expr = ($sclass ? "-sclass $sclass" : "");
    my $cmd = "$CFG::shipping_server -poll $sclass_expr";
    my $res = Do_System($cmd, __FILE__, __LINE__);
    if ($res) {
        print2 "ERROR: error executing \"$cmd\" - continuing.\n";
        $CFG::failcount++;
    }
}

# use this array to collect names of all reps to export, along with
# params for that particular export. After parsing all cmd line and
# script-file stuff, then go ahead and start sending packets.
my @PktsToSend;
my $NumPkts = 0;

# if "-all" specified, do all local replicas
if ($all_local) {
    my @vobs = GetLocalVobs();
    dbgprint "localvobs: " . join(" ", @vobs) . "\n";

    foreach my $vob (@vobs) {
        my @siblings = GetSiblings($vob);
        $export_vob = $vob;
        foreach my $sib (@siblings) {
            $reptosend = $sib;

            $PktsToSend[$NumPkts++] = CollectPacket();
            dbgprint "CollectPkt: " . CollectPacket() . "\n";
        }
    }
}

# sync any replicas specified on cmd line
my $rep;
foreach $rep (@replicas) {
    if ($export_vob) {
        $reptosend = $rep;
        $PktsToSend[$NumPkts++] = CollectPacket();
    } else {
        print2 "ERROR: no VOB specified for replica \"$rep\".\n";
        $CFG::failcount++;
    }
}

if ($scriptfile && !(-e $scriptfile)) {
    print2 "ERROR: Script file $scriptfile does not exist\n";
    $scriptfile = '';
    $CFG::failcount++;
}

if ($scriptfile) {
    my ($tag, $val);
    dbgprint "opening export script '$scriptfile'.\n";
    open (SCR, "<$scriptfile");
    while (<SCR>) {
        chomp;
        ($tag, $val) = split(":", $_);
        dbgprint "scriptfile - tag: $tag value: $val\n";
        @replicas = ();
        if ("replicas" =~ m/^$tag/) {
            dbgprint "found replicas...\n";
            ($export_vob,@replicas) = GetReplicas($val);
        } elsif ("maxsize" =~ m/^$tag/) {
            dbgprint "found maxsize...\n";
            $maxsize = $val;
        } elsif ("sclass" =~ m/^$tag/) {
            dbgprint "found sclass...\n";
            $sclass = $val;
        } elsif ("workdir" =~ m/^$tag/) {
            dbgprint "found workdir...\n";
            $workdir = $val;
        } elsif ("nocompress" =~ m/^$tag/) {
            dbgprint "found nocompress...\n";
            $do_compress = 0;
        } elsif ("compress" =~ m/^$tag/) {
            dbgprint "found compress...\n";
            $do_compress = 1;
        } elsif ("update" =~ m/^$tag/) {
            dbgprint "found update...\n";
            $updateflg = 1;
        } elsif ("noupdate" =~ m/^$tag/) {
            dbgprint "found noupdate...\n";
            $updateflg = 0;
        } elsif ("ship" =~ m/^$tag/) {
            dbgprint "found ship...\n";
            $fshipflg = 0;
            $shipflg = 1;
        } elsif ("limit" =~ m/^$tag/) {
            dbgprint "found limit...\n";
            $limit = $val;
        } elsif ("fship" =~ m/^$tag/) {
            dbgprint "found fship...\n";
            $fshipflg = 1;
            $shipflg = 0;
        } elsif ("lockwait" =~ m/^$tag/) {
            dbgprint "found lockwait...\n";
            $lockwait = $val;
        } elsif ("nblocking" =~ m/^$tag/) {
            dbgprint "found nblocking...\n";
            $nblocking = 1;
        } elsif ("nonblocking" =~ m/^$tag/) {
            dbgprint "found nonblocking...\n";
            $nblocking = 0;
        } elsif ("view" =~ m/^$tag/) {
            dbgprint "found view...\n";
            $view = $val;
        } else {
            print2 "ERROR: unrecognized option '$tag $val' in " .
                "scriptfile $scriptfile.\n";
            $CFG::failcount++;
        }
        #
        # if we found a 'replicas' tag in the file, start export processing
        #
	foreach $rep (@replicas) {
	    if ($export_vob) {
                $reptosend = $rep;
		$PktsToSend[$NumPkts++] = CollectPacket();
	    } else {
		print2 "WARNING: No sync done; no VOB specified for " .
                    "replica \"$rep\".\n";
	    }
	}
    }
}

my $options_used = "command options specified or defaulted:\n" .
    "    compress: $do_compress\n" .
    "    logdir: $logdir\n    storage-class: $sclass\n" . 
    "    workdir: $workdir\n" .
    "    maxsize: $maxsize\n    limit: $limit\n    all: $all_local\n" .
    "    fship: $fshipflg\n    ship: $shipflg\n    poll: $poll\n" .
    "    lockwait: $lockwait minutes\n" .
    "    view: $view\n    nblocking: $nblocking\n" .
    "    retries: $num_tries times, wait $sleeptime seconds\n" .
    "    script: $scriptfile\n";

print $options_used if ($CFG::verbose || $CFG::traceflg);

print STDERR $options_used if ($CFG::verbose || $CFG::traceflg);

while ($num_tries--) {
    # this perl 'grep' cmd takes the list @PktsToSend and passes each element
    # into the {block} as the temp variable '$_'. The grep returns a new list
    # containing only the packets that were NOT successfully processed,
    # which replaces the previous @PktsToSend.
    @PktsToSend = grep {
        my $res = SendPacket($_);
        if ($res) {
            $CFG::failcount++;
        }
        $res;
    } @PktsToSend;    # end of the 'grep'

    dbgprint "packets left: " . (scalar @PktsToSend) . 
        " tries left: $num_tries\n";
    if ($num_tries && scalar @PktsToSend) {
         my $num = scalar @PktsToSend;
        print2 "$num packets not exported; waiting " .
            "$sleeptime seconds to re-try.\n";
         sleep $sleeptime;
    }
}

my ($cmd, $res);
my $pkts_left = scalar @PktsToSend;
if ($CFG::failcount && $pkts_left) {
    if ($CFG::NT) { # only do event log for NT
	$cmd = "$CFG::imsglog $pkg_name evtlog \"$logname\"";
	$res = Do_System($cmd, __FILE__, __LINE__);
    }
    print2 "\n\n\nERROR: $CFG::failcount error(s) encountered during " .
        "export processing.\n$pkts_left packet(s) may not have been " .
        "sent successfully. See log at:\n    \"$logname\".\n\n\n";
    close STDERR;
} else {
    if ($CFG::NT) { # only do event log for NT
	$cmd = "$CFG::imsglog $pkg_name evtlog";
	$res = Do_System($cmd, __FILE__, __LINE__);
    }
    if ($CFG::failcount) {
        # there's an error count, but no packets are left. The errors must
        # have been only temporary. Send out a msg to explain this.
        print2 "\nINFO: $CFG::failcount temporary errors were " .
            "encountered, but all\n    packets were successfully exported.\n";
    }
    if ($CFG::verbose || $CFG::traceflg || $CFG::failcount) {  # keep log file
        qprint "\n\nSUCCESSFUL COMPLETION: " .
            "see log file at:\n    \"$logname\".\n\n";
        print STDERR "\nSUCCESSFUL COMPLETION of export processing.\n";
        close STDERR;
    } else {
        qprint "\n\nSUCCESSFUL COMPLETION: log file removed.\n\n";
        close STDERR;
        unlink $logname;
    }
}

close STDOUT;
exit $CFG::failcount;

sub CollectPacket {
    # concat all SendPacket() params into single entity to store in array.
    return join(";", ($export_vob, $reptosend, $do_compress, $maxsize, 
                $limit, $sclass, $fshipflg, $updateflg, $lockwait,
                $view, $nblocking));
}

sub UnpackArgs {
    # reverse of the CollectPacket() function.
    return split(";", shift);
}

sub SendPacket {
    #
    # Send the replica synchronization packet
    #


    dbgprint "args passed to SendPacket() " . $_ . "\n";
    my ($export_vob, $replica, $do_compress, $maxsize, $limit,
        $sclass, $fshipflg, $updateflg, $lockwait, 
        $view, $nblocking) = UnpackArgs(shift);

    my ($cmd, $res);
    my $rep_sel = "replica:$replica\@$export_vob";

    # use 'chepoch -actual' to update local replica's epoch table before
    # starting the export if requested.
    if ($updateflg) {
        my $cmd = "$CFG::MultiTool chepoch -actual $replica\@$export_vob >&2";
        my $res = Do_System($cmd, __FILE__, __LINE__);
        if ($res) {
            print2 "WARNING: error attempting to update epoch table " .
                "from $replica\@$export_vob. Continuing.\n";
        }
    }

    my $sclass_expr = ($sclass ? "-sclass $sclass" : "");
    my $max_expr = ($maxsize ? "-maxsize $maxsize" : '');
    my $limit_expr = ($limit ? "-limit $limit" : "");
    my $ship_expr = ($fshipflg ? "-fship" : "-ship");
    my $compress_expr = ($do_compress ? "-compress" : "");
    my $view_expr = ($view ? "-view $view" : "");
    my $wd_expr = ($workdir ? "-workdir $workdir" : "");
    my $lw_expr = ($lockwait ? "-lockwait $lockwait" : "");
    my $other_expr = $ENV{MULTISITE_EXPORT_OTHER_PARAMS};

    my $block_expr = ($nblocking ? "-nblocking" : "-wait_forever");

    $cmd = "$multitool_cmd $max_expr $ship_expr $block_expr $view_expr " .
	"$limit_expr $compress_expr $sclass_expr $other_expr " .
        "$wd_expr $lw_expr $rep_sel >&2";

    $res = Do_System($cmd, __FILE__, __LINE__);
    if ($res) {
        print2 "ERROR: command '$cmd' encountered error.\n";
	# if the sync failed:  Update the event log with
	# a pointer to the log file.
	#
	dbgprint qq/ERROR: MultiSite export error; see logfile in "$logname"\n/;
	return 1;
    }

    return 0;
}

sub GetArchitecture {
    # figure out architecture we're on. Config.pm package and $^O Perl variable
    # both have changed depending on the version of perl or ccperl that actually
    # runs the script, so we need to try a couple of combinations.

    # This should go in syncmgr_common.pl, EXCEPT that we need to figure out the
    # architecture and know the PATH to syncmgr_common.pl before we can
    # 'require' it.

    my $arch = $^O ? $^O : $main::Config{archname};

    my $isNT = (($arch =~ /^MSWin32/) || ($arch eq 'i386-win32')) ? 1 : 0;
    my $isUNIX = ($CFG::NT ? 0 : 1);

    return ($isNT, $isUNIX);
}

__END__
:endofperl
ccperl -S ccperlpp.pl %0.bat %*

